@@ -850,11 +850,24 @@ s2n_modemstate(net_info_t *netcon, struct gensio *io, unsigned int modemstate)
     gensiods len;
 
     netcon->modemstate_mask = modemstate;
+#ifdef GENSIO_ACONTROL_SER_SET_MODEMSTATE_MASK
+    len = snprintf(s, sizeof(s), "%d", modemstate);
+    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_SET_MODEMSTATE_MASK, s, len,
+		    NULL, NULL, NULL);
+    len = snprintf(s, sizeof(s), "%d",
+		   port->last_modemstate & netcon->modemstate_mask);
+    gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET,
+		   GENSIO_CONTROL_SER_SEND_MODEMSTATE, s, &len);
+#else
     len = snprintf(s, sizeof(s), "%d",
 		   port->last_modemstate & netcon->modemstate_mask);
     gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
 		   GENSIO_CONTROL_SET,
 		   GENSIO_CONTROL_SER_MODEMSTATE, s, &len);
+#endif
 }
 
 static void
@@ -865,11 +878,24 @@ s2n_linestate(net_info_t *netcon, struct gensio *io, unsigned int linestate)
     gensiods len;
 
     netcon->linestate_mask = linestate;
+#ifdef GENSIO_ACONTROL_SER_SET_LINESTATE_MASK
+    len = snprintf(s, sizeof(s), "%d", linestate);
+    gensio_acontrol(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		    GENSIO_CONTROL_SET,
+		    GENSIO_ACONTROL_SER_SET_LINESTATE_MASK, s, len,
+		    NULL, NULL, NULL);
+    len = snprintf(s, sizeof(s), "%d",
+		   port->last_linestate & netcon->linestate_mask);
+    gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
+		   GENSIO_CONTROL_SET,
+		   GENSIO_CONTROL_SER_SEND_LINESTATE, s, &len);
+#else
     len = snprintf(s, sizeof(s), "%d",
 		   port->last_linestate & netcon->linestate_mask);
     gensio_control(netcon->net, GENSIO_CONTROL_DEPTH_FIRST,
 		   GENSIO_CONTROL_SET,
 		   GENSIO_CONTROL_SER_LINESTATE, s, &len);
+#endif
 }
 
 static void
@@ -884,13 +910,37 @@ s2n_flowcontrol_state(net_info_t *netcon, bool val)
 		   GENSIO_CONTROL_SER_FLOWCONTROL_STATE, s, &len);
 }
 
+#ifndef GENSIO_SER_FLUSH_BOTH
+/* This was missed in versions of gensio before 2.8.2. */
+const char *
+gensio_flush_to_str(unsigned int ival)
+{
+    switch(ival) {
+    case 0:
+	return "0";
+
+    case 1:
+	return "recv";
+
+    case 2:
+	return "xmit";
+
+    case 3:
+	return "both";
+
+    default:
+	return "?";
+    }
+}
+#endif
+
 static void
 s2n_flush(net_info_t *netcon, int val)
 {
     char s[20];
     gensiods len;
 
-    strncpy(s, gensio_onoff_to_str(val), sizeof(s) - 1);
+    strncpy(s, gensio_flush_to_str(val), sizeof(s) - 1);
     len = strlen(s);
 
     if (netcon->port->io)
@@ -977,14 +1027,11 @@ s2n_flowcontrol(net_info_t *netcon, int flowcontrol)
 
     if (!io)
 	return;
-    printf("A: %d\n", flowcontrol);
     len = snprintf(s, sizeof(s), "%s", gensio_flowcontrol_to_str(flowcontrol));
-    printf("B: %ld %s\n", len, s);
-    int rv = gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
+    gensio_acontrol(io, GENSIO_CONTROL_DEPTH_FIRST,
 		    GENSIO_CONTROL_SET,
 		    GENSIO_ACONTROL_SER_FLOWCONTROL, s, len, ser_control_set,
 		    (void *) (long) S2N_FLOWCONTROL, NULL);
-    printf("C: %d\n", rv);
 }
 
 static void
