@@ -67,10 +67,8 @@ static struct default_data defaults[] = {
 					.def.intval = 10 },
     { "remaddr",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
     { "connback",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
-    { "authdir",	GENSIO_DEFAULT_STR,	.def.strval =
-						DATAROOT "/ser2net/auth" },
-    { "authdir-admin",	GENSIO_DEFAULT_STR,	.def.strval =
-						SYSCONFDIR "/ser2net/auth" },
+    { "authdir",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
+    { "authdir-admin",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
     { "pamauth",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
     { "pamauth-admin",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
     { "allowed-users",	GENSIO_DEFAULT_STR,	.def.strval = NULL },
@@ -103,12 +101,16 @@ setup_ser2net_defaults(void)
 	if (err)
 	    return err;
     }
-    err = gensio_set_default(so, "ssl", "key",
-			     SYSCONFDIR "/ser2net/ser2net.key", 0);
+    err = gensio_set_default(so, NULL, "authdir", authdir, 0);
+    if (err)
+      return err;
+    err = gensio_set_default(so, NULL, "authdir-admin", admin_authdir, 0);
+    if (err)
+      return err;
+    err = gensio_set_default(so, "ssl", "key", keyfile, 0);
     if (err)
 	return err;
-    err = gensio_set_default(so, "ssl", "cert",
-			     SYSCONFDIR "/ser2net/ser2net.crt", 0);
+    err = gensio_set_default(so, "ssl", "cert", certfile, 0);
     if (err)
 	return err;
 #ifdef gensio_version_ge /* gensio_version_ge came in with 2.2.3 and 2.3.0 */
