@@ -24,7 +24,6 @@
  */
 
 #include <stdlib.h>
-#include <errno.h>
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
@@ -41,8 +40,6 @@
 /* When the version info was added, the type was changed. */
 typedef struct gensio_addr gaddrinfo;
 #define gensio_free_addrinfo(o, a) gensio_addr_free(a)
-#include <sys/socket.h>
-#include <netdb.h>
 #else
 typedef struct addrinfo gaddrinfo;
 #endif
@@ -221,7 +218,7 @@ port_add_remaddr(struct absout *eout, port_info_t *port, const char *istr)
     str = strdup(istr);
     if (!str) {
 	eout->out(eout, "Out of memory handling remote address '%s'", istr);
-	return ENOMEM;
+	return GE_NOMEM;
     }
 
     remstr = strtok_r(str, ";", &strtok_data);
@@ -251,7 +248,7 @@ port_add_connback(struct absout *eout, port_info_t *port, const char *istr)
     if (!str) {
 	eout->out(eout, "Out of memory handling connect back address '%s'",
 		  istr);
-	return ENOMEM;
+	return GE_NOMEM;
     }
 
     remstr = strtok_r(str, ";", &strtok_data);
@@ -332,17 +329,12 @@ finish_free_port(port_info_t *port)
 	free(port->orig_devname);
     if (port->sendon)
 	free(port->sendon);
+    if (port->led_rx)
+	free_led(port->led_rx);
+    if (port->led_tx)
+	free_led(port->led_tx);
 #ifdef DO_MDNS
-    if (port->mdns_name)
-	free(port->mdns_name);
-    if (port->mdns_type)
-	free(port->mdns_type);
-    if (port->mdns_domain)
-	free(port->mdns_domain);
-    if (port->mdns_host)
-	free(port->mdns_host);
-    if (port->mdns_txt)
-	gensio_argv_free(so, port->mdns_txt);
+    mdns_shutdown(&port->mdns_info);
 #endif /* DO_MDNS */
     free(port);
 }
@@ -398,7 +390,7 @@ strdupcat(char **str, const char *cat)
     char *s = malloc(strlen(*str) + strlen(cat) + 2);
 
     if (!s)
-	return ENOMEM;
+	return GE_NOMEM;
     strcpy(s, *str);
     strcat(s, ",");
     strcat(s, cat);
@@ -407,42 +399,6 @@ strdupcat(char **str, const char *cat)
     return 0;
 }
 
-static const char *serialdev_parms[] = {
-    "XONXOFF", NULL,
-    "-XONXOFF", "xonxoff=false",
-    "RTSCTS", NULL,
-    "-RTSCTS", "rtscts=false",
-    "LOCAL", NULL,
-    "-LOCAL", "local=false",
-    "HANGUP_WHEN_DONE", "hangup-when-done",
-    "-HANGUP_WHEN_DONE", "hangup-when-done=false",
-    "NOBREAK", NULL,
-    "-NOBREAK", "nobreak=false",
-    "NONE", NULL,
-    "EVEN", NULL,
-    "ODD", NULL,
-    "MARK", NULL,
-    "SPACE", NULL,
-    NULL
-};
-
-static bool
-matchstr(const char *parms[], const char *c, const char **newval)
-{
-    unsigned int i;
-
-    for (i = 0; parms[i]; i += 2) {
-	if (strcmp(parms[i], c) == 0) {
-	    if (parms[i + 1])
-		*newval = parms[i + 1];
-	    else
-		*newval = parms[i];
-	    return true;
-	}
-    }
-    return false;
-}
-
 static int
 check_keyvalue_default(const char *str, const char *name, const char **value,
 		       const char *def)
@@ -470,37 +426,15 @@ update_str_val(const char *str, char **outstr, const char *name,
     return 0;
 }
 
-#ifdef DO_MDNS
-static struct gensio_enum_val mdns_nettypes[] = {
-    { "unspec", GENSIO_NETTYPE_UNSPEC },
-    { "ipv4", GENSIO_NETTYPE_IPV4 },
-    { "ipv6", GENSIO_NETTYPE_IPV6 },
-    { NULL }
-};
-#endif /* DO_MDNS */
-
 static int
 myconfig(port_info_t *port, struct absout *eout, const char *pos)
 {
-    enum str_type stype;
-    char *s, *fval;
-    const char *val, *newval = pos;
-    unsigned int len;
+    char *fval;
+    const char *val;
     int rv;
 
-    /*
-     * This is a hack for backwards compatibility, if we see a config
-     * item meant for the device, we stick it onto the device name.
-     */
-    if (isdigit(pos[0]) || matchstr(serialdev_parms, pos, &newval)) {
-	int err = strdupcat(&port->devname, newval);
-
-	if (err) {
-	    eout->out(eout, "Out of memory appending to devname");
-	    return -1;
-	}
-    } else if (gensio_check_keybool(pos, "kickolduser",
-				    &port->kickolduser_mode) > 0) {
+    if (gensio_check_keybool(pos, "kickolduser",
+			     &port->kickolduser_mode) > 0) {
     } else if (gensio_check_keybool(pos, "trace-hexdump",
 				    &port->trace_read.hexdump) > 0) {
 	port->trace_write.hexdump = port->trace_read.hexdump;
@@ -612,6 +546,9 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 	rv = port_add_connback(eout, port, val);
 	if (rv)
 	    return -1;
+    } else if (gensio_check_keyuint(pos, "connback-timeout",
+				    &port->connback_timeout) > 0) {
+	port->connback_timeout_set = true;
     } else if (check_keyvalue_default(pos, "banner", &val, "") > 0) {
 	fval = strdup(val);
 	if (!fval) {
@@ -643,7 +580,7 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 	struct timeval tv = { 0, 0 };
 	gensiods len;
 
-	fval = process_str_to_str(port, NULL, val, &tv, &len, false);
+	fval = process_str_to_str(port, NULL, val, &tv, &len, false, eout);
 	if (!fval) {
 	    eout->out(eout, "Out of memory allocating closeon");
 	    return -1;
@@ -665,7 +602,7 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 	struct timeval tv =  { 0, 0 };
 	gensiods len;
 
-	fval= process_str_to_str(port, NULL, val, &tv, &len, false);
+	fval= process_str_to_str(port, NULL, val, &tv, &len, false, eout);
 	if (!fval) {
 	    eout->out(eout, "Out of memory allocating sendon");
 	    return -1;
@@ -675,124 +612,9 @@ myconfig(port_info_t *port, struct absout *eout, const char *pos)
 	port->sendon = fval;
 	port->sendon_len = len;
 #ifdef DO_MDNS
-    } else if (gensio_check_keybool(pos, "mdns",
-				    &port->mdns) > 0) {
-    } else if (gensio_check_keybool(pos, "mdns-sysattrs",
-				    &port->do_mdns_sysattrs) > 0) {
-    } else if (gensio_check_keyuint(pos, "mdns-port",
-				    &port->mdns_port) > 0) {
-    } else if (gensio_check_keyint(pos, "mdns-interface",
-				   &port->mdns_interface) > 0) {
-    } else if (gensio_check_keyenum(pos, "mdns-nettype",
-				    mdns_nettypes,
-				    &port->mdns_nettype) > 0) {
-    } else if (gensio_check_keyvalue(pos, "mdns-name", &val) > 0) {
-	fval = strdup(val);
-	if (!fval) {
-	    eout->out(eout, "Out of memory allocating mdns-name");
-	    return -1;
-	}
-	if (port->mdns_name)
-	    free(port->mdns_name);
-	port->mdns_name = fval;
-    } else if (gensio_check_keyvalue(pos, "mdns-type", &val) > 0) {
-	fval = strdup(val);
-	if (!fval) {
-	    eout->out(eout, "Out of memory allocating mdns-type");
-	    return -1;
-	}
-	if (port->mdns_type)
-	    free(port->mdns_type);
-	port->mdns_type = fval;
-    } else if (gensio_check_keyvalue(pos, "mdns-domain", &val) > 0) {
-	fval = strdup(val);
-	if (!fval) {
-	    eout->out(eout, "Out of memory allocating mdns-domain");
-	    return -1;
-	}
-	if (port->mdns_domain)
-	    free(port->mdns_domain);
-	port->mdns_domain = fval;
-    } else if (gensio_check_keyvalue(pos, "mdns-host", &val) > 0) {
-	fval = strdup(val);
-	if (!fval) {
-	    eout->out(eout, "Out of memory allocating mdns-host");
-	    return -1;
-	}
-	if (port->mdns_host)
-	    free(port->mdns_host);
-	port->mdns_host = fval;
-    } else if (gensio_check_keyvalue(pos, "mdns-txt", &val) > 0) {
-	int err = gensio_argv_append(so, &port->mdns_txt, val,
-				     &port->mdns_txt_args, &port->mdns_txt_argc,
-				     true);
-
-	if (err) {
-	    eout->out(eout, "Out of memory allocating mdns-txt: %s",
-		      gensio_err_to_str(err));
-	    return -1;
-	}
+    } else if (mdns_checkoption(pos, &port->mdns_info, port->name, eout) > 0) {
 #endif /* DO_MDNS */
-
-    /* Everything from here down to the banner, etc is deprecated. */
-    } else if (strcmp(pos, "remctl") == 0) {
-	port->allow_2217 = true;
-    } else if (strcmp(pos, "-remctl") == 0) {
-	port->allow_2217 = false;
-    } else if (strcmp(pos, "-kickolduser") == 0) {
-        port->kickolduser_mode = 0;
-    } else if (strcmp(pos, "-hexdump") == 0) {
-	port->trace_read.hexdump = false;
-	port->trace_write.hexdump = false;
-	port->trace_both.hexdump = false;
-    } else if (strcmp(pos, "-timestamp") == 0) {
-	port->trace_read.timestamp = false;
-	port->trace_write.timestamp = false;
-	port->trace_both.timestamp = false;
-    } else if (strcmp(pos, "-tr-hexdump") == 0) {
-	port->trace_read.hexdump = false;
-    } else if (strcmp(pos, "-tr-timestamp") == 0) {
-	port->trace_read.timestamp = false;
-    } else if (strcmp(pos, "-tw-hexdump") == 0) {
-	port->trace_write.hexdump = false;
-    } else if (strcmp(pos, "-tw-timestamp") == 0) {
-	port->trace_write.timestamp = false;
-    } else if (strcmp(pos, "-tb-hexdump") == 0) {
-	port->trace_both.hexdump = false;
-    } else if (strcmp(pos, "-tb-timestamp") == 0) {
-	port->trace_both.timestamp = false;
-    } else if (gensio_check_keyvalue(pos, "tr", &val) > 0) {
-	/* trace read, data from the port to the socket */
-	port->trace_read.filename = find_tracefile(val);
-    } else if (gensio_check_keyvalue(pos, "tw", &val) > 0) {
-	/* trace write, data from the socket to the port */
-	port->trace_write.filename = find_tracefile(val);
-    } else if (gensio_check_keyvalue(pos, "tb", &val) > 0) {
-	/* trace both directions. */
-	port->trace_both.filename = find_tracefile(val);
-    } else if (gensio_check_keyvalue(pos, "rs485", &val) > 0) {
-	port->rs485 = find_rs485conf(val);
-    } else if (strcmp(pos, "telnet_brk_on_sync") == 0) {
-	port->telnet_brk_on_sync = 1;
-    } else if (strcmp(pos, "-telnet_brk_on_sync") == 0) {
-	port->telnet_brk_on_sync = 0;
-    } else if (strcmp(pos, "-chardelay") == 0) {
-	port->enable_chardelay = false;
-
-    /* Banner and friend handling. */
-    } else if ((s = find_str(pos, &stype, &len))) {
-	/* It's a startup banner, signature or open/close string, it's
-	   already set. */
-	switch (stype) {
-	case BANNER: port->bannerstr = s; break;
-	case SIGNATURE: port->signaturestr = s; break;
-	case OPENSTR: port->openstr = s; break;
-	case CLOSESTR: port->closestr = s; break;
-	case CLOSEON: port->closeon = s; port->closeon_len = len; break;
-	default: free(s); goto unknown;
-	}
     } else {
-    unknown:
 	eout->out(eout, "Unknown config item: %s", pos);
 	return -1;
     }
@@ -822,15 +644,15 @@ process_connect_back(struct absout *eout, port_info_t *port,
 }
 
 static int
-init_port_data(port_info_t *port)
+init_port_data(port_info_t *port, struct absout *eout)
 {
     port->enabled = false;
 
     port->net_to_dev_state = PORT_CLOSED;
     port->dev_to_net_state = PORT_CLOSED;
-    port->trace_read.fd = -1;
-    port->trace_write.fd = -1;
-    port->trace_both.fd = -1;
+    port->trace_read.f = NULL;
+    port->trace_write.f = NULL;
+    port->trace_both.f = NULL;
 
     port->telnet_brk_on_sync = find_default_bool("telnet-brk-on-sync");
     port->kickolduser_mode = find_default_bool("kickolduser");
@@ -844,37 +666,31 @@ init_port_data(port_info_t *port)
     port->connector_retry_time = find_default_int("connector-retry-time");
     port->accepter_retry_time = find_default_int("accepter-retry-time");
     if (find_default_str("authdir", &port->authdir))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("pamauth", &port->pamauth))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("allowed-users", &port->default_allowed_users))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("signature", &port->signaturestr))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("banner", &port->bannerstr))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("openstr", &port->openstr))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("closestr", &port->closestr))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("closeon", &port->closeon))
-	return ENOMEM;
+	return GE_NOMEM;
     if (find_default_str("sendon", &port->sendon))
-	return ENOMEM;
+	return GE_NOMEM;
 
     port->led_tx = NULL;
     port->led_rx = NULL;
 
 #ifdef DO_MDNS
-    port->mdns = find_default_bool("mdns");
-    port->do_mdns_sysattrs = find_default_bool("mdns-sysattrs");
-    port->mdns_interface = find_default_int("mdns-interface");
-    if (find_default_str("mdns-type", &port->mdns_type))
-	return ENOMEM;
-    if (find_default_str("mdns-domain", &port->mdns_domain))
-	return ENOMEM;
-    if (find_default_str("mdns-host", &port->mdns_host))
-	return ENOMEM;
+    mdns_info_getdefaults(&port->mdns_info, port->name, eout);
+    /* The bool is not defaulted in getdefaults. */
+    port->mdns_info.mdns = find_default_bool("mdns");
 #endif /* DO_MDNS */
 
     return 0;
@@ -892,7 +708,6 @@ portconfig(struct absout *eout,
 {
     port_info_t *new_port, *curr;
     net_info_t *netcon;
-    enum str_type str_type;
     int err;
     bool write_only = false;
     unsigned int i;
@@ -924,27 +739,15 @@ portconfig(struct absout *eout,
 	goto errout;
     }
 
-    new_port->devname = find_str(devname, &str_type, NULL);
-    if (new_port->devname) {
-	if (str_type != DEVNAME) {
-	    free(new_port->devname);
-	    new_port->devname = NULL;
-	} else {
-	    new_port->orig_devname = strdup(devname);
-	    if (!new_port->orig_devname) {
-		eout->out(eout, "unable to allocate original device name");
-		goto errout;
-	    }
-	}
-    }
-    if (!new_port->devname)
-	new_port->devname = strdup(devname);
+    new_port->devname = strdup(devname);
     if (!new_port->devname) {
 	eout->out(eout, "unable to allocate device name");
 	goto errout;
     }
 
-    init_port_data(new_port);
+    err = init_port_data(new_port, eout);
+    if (err)
+	goto errout;
 
     if (!new_port->name) {
 	new_port->name = strdup(name);
