@@ -231,11 +231,10 @@ char *rfc2217_signature = "ser2net";
 static char *help_string =
 "%s: Valid parameters are:\n"
 "  -c <config file> - use a config file besides %s\n"
-"  -C <config line> - Handle a single configuration line.  This may be\n"
-"     specified multiple times for multiple lines.  This is just like a\n"
-"     line in the config file.  This disables the default config file,\n"
-"     you must specify a -c after the last -C to have it read a config\n"
-"     file, too.  The config file must not be yaml.\n"
+"  -C <config line> - Set the directory where the configuration file and\n"
+"     ssl keys are stored.  This is generally /etc/ser2net on Unix-type\n"
+"     systems, on Windows it is ../etc/ser2net from the executable's\n"
+"     location by default.\n"
 "  -p <controller port> - Start a controller session on the given TCP port\n"
 "  -P <file> - set location of pid file\n"
 "  -n - Don't detach from the controlling terminal\n"
@@ -921,7 +920,7 @@ main(int argc, char *argv[])
 
 	case 'v':
 	    printf("%s version %s\n", argv[0], VERSION);
-	    return 1;
+	    return 0;
 
 	case 's':
             i++;
