@@ -336,39 +336,43 @@ def test_dtr():
 
     o = utils.o
     ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
-    sio1 = io1.cast_to_sergensio()
-    io1.handler.set_compare("12345")
-    if (io1.handler.wait_timeout(1000) == 0):
-        raise Exception("%s: %s: Timed out waiting for banner" %
-                        ("test dtr", io1.handler.name))
+    try:
+        sio1 = io1.cast_to_sergensio()
+        io1.handler.set_compare("12345")
+        if (io1.handler.wait_timeout(1000) == 0):
+            raise Exception("%s: %s: Timed out waiting for banner" %
+                            ("test dtr", io1.handler.name))
 
-    io1.read_cb_enable(True);
-    io2.read_cb_enable(True);
+        io1.read_cb_enable(True);
+        io2.read_cb_enable(True);
 
-    set_remote_null_modem(utils.remote_id_int(io2), False);
+        set_remote_null_modem(utils.remote_id_int(io2), False);
 
-    val = sio1.sg_dtr_s(0)
-    if (val != gensio.SERGENSIO_DTR_ON):
-        raise Exception("Expected DTR on at start, got %d" % val);
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (not (val & SERIALSIM_TIOCM_DTR)):
-        raise Exception("Expected remote DTR on at start");
+        val = sio1.sg_dtr_s(0)
+        if (val != gensio.SERGENSIO_DTR_ON):
+            raise Exception("Expected DTR on at start, got %d" % val);
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (not (val & SERIALSIM_TIOCM_DTR)):
+            raise Exception("Expected remote DTR on at start");
 
-    val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_OFF)
-    if (val != gensio.SERGENSIO_DTR_OFF):
-        raise Exception("Expected DTR off");
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (val & SERIALSIM_TIOCM_DTR):
-        raise Exception("Expected remote DTR off");
+        val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_OFF)
+        if (val != gensio.SERGENSIO_DTR_OFF):
+            raise Exception("Expected DTR off");
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (val & SERIALSIM_TIOCM_DTR):
+            raise Exception("Expected remote DTR off");
 
-    val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_ON)
-    if (val != gensio.SERGENSIO_DTR_ON):
-        raise Exception("Expected DTR on");
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (not (val & SERIALSIM_TIOCM_DTR)):
-        raise Exception("Expected remote DTR on");
+        val = sio1.sg_dtr_s(gensio.SERGENSIO_DTR_ON)
+        if (val != gensio.SERGENSIO_DTR_ON):
+            raise Exception("Expected DTR on");
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (not (val & SERIALSIM_TIOCM_DTR)):
+            raise Exception("Expected remote DTR on");
 
-    set_remote_null_modem(utils.remote_id_int(io2), True);
+        set_remote_null_modem(utils.remote_id_int(io2), True);
+    except:
+        utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
+        raise
     utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
     print("  Success!")
     return
@@ -389,45 +393,107 @@ def test_rts():
 
     o = utils.o
     ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
-    sio1 = io1.cast_to_sergensio()
-    io1.handler.set_compare("12345")
-    if (io1.handler.wait_timeout(1000) == 0):
-        raise Exception("%s: %s: Timed out waiting for banner" %
-                        (io1.handler.name))
 
-    io1.read_cb_enable(True);
-    io2.read_cb_enable(True);
+    try:
+        sio1 = io1.cast_to_sergensio()
+        io1.handler.set_compare("12345")
+        if (io1.handler.wait_timeout(1000) == 0):
+            raise Exception("%s: %s: Timed out waiting for banner" %
+                            (io1.handler.name))
 
-    set_remote_null_modem(utils.remote_id_int(io2), False);
+        io1.read_cb_enable(True);
+        io2.read_cb_enable(True);
 
-    val = sio1.sg_rts_s(0)
-    if (val != gensio.SERGENSIO_RTS_ON):
-        raise Exception("Expected RTS on at start, got %d" % val);
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (not (val & SERIALSIM_TIOCM_RTS)):
-        raise Exception("Expected remote RTS on at start");
+        set_remote_null_modem(utils.remote_id_int(io2), False);
 
-    val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_OFF)
-    if (val != gensio.SERGENSIO_RTS_OFF):
-        raise Exception("Expected RTS off");
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (val & SERIALSIM_TIOCM_RTS):
-        raise Exception("Expected remote RTS off");
+        val = sio1.sg_rts_s(0)
+        if (val != gensio.SERGENSIO_RTS_ON):
+            raise Exception("Expected RTS on at start, got %d" % val);
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (not (val & SERIALSIM_TIOCM_RTS)):
+            raise Exception("Expected remote RTS on at start");
 
-    val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_ON)
-    if (val != gensio.SERGENSIO_RTS_ON):
-        raise Exception("Expected RTS on");
-    val = get_remote_modem_ctl(utils.remote_id_int(io2))
-    if (not (val & SERIALSIM_TIOCM_RTS)):
-        raise Exception("Expected remote RTS on");
+        val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_OFF)
+        if (val != gensio.SERGENSIO_RTS_OFF):
+            raise Exception("Expected RTS off");
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (val & SERIALSIM_TIOCM_RTS):
+            raise Exception("Expected remote RTS off");
 
-    set_remote_null_modem(utils.remote_id_int(io2), True);
+        val = sio1.sg_rts_s(gensio.SERGENSIO_RTS_ON)
+        if (val != gensio.SERGENSIO_RTS_ON):
+            raise Exception("Expected RTS on");
+        val = get_remote_modem_ctl(utils.remote_id_int(io2))
+        if (not (val & SERIALSIM_TIOCM_RTS)):
+            raise Exception("Expected remote RTS on");
+
+        set_remote_null_modem(utils.remote_id_int(io2), True);
+    except:
+        utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
+        raise
     utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
     print("  Success!")
     return
 
 test_rts()
 
+class CtrlRspHandler:
+    def __init__(self, o, val):
+        self.val = val
+        self.waiter = gensio.waiter(o)
+        return
+
+    def control_done(self, io, err, value):
+        if (err):
+            raise Exception("Error getting signature: %s" % err)
+        value = value.decode(encoding='utf-8')
+        if (value != str(self.val)):
+            raise Exception("Value was '%s', expected '%s'" %
+                            (value, self.val))
+        self.waiter.wake();
+        return
+
+    def wait_timeout(self, timeout):
+        return self.waiter.wait_timeout(1, timeout)
+
+def test_flush():
+    config = ("connection: &con",
+              "  accepter: telnet(rfc2217),tcp,3023",
+              "  connector: serialdev,/dev/ttyPipeA0,9600n81,local")
+    io1str = "telnet(rfc2217),tcp,localhost,3023"
+    io2str = "serialdev,/dev/ttyPipeB0,9600N81,local"
+
+    print("serialdev flush rfc2217:\n  config=%s  io1=%s\n  io2=%s" %
+          (config, io1str, io2str))
+
+    o = utils.o
+    ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str)
+
+    try:
+        gensio.GENSIO_ACONTROL_SER_FLUSH
+    except:
+        print("  Unsupported!")
+        return
+
+    try:
+        io1.read_cb_enable(True);
+        io1.handler.set_expected_flush(1)
+        sio1 = io1.cast_to_sergensio()
+        h = CtrlRspHandler(o, "recv")
+        io1.acontrol(0, gensio.GENSIO_CONTROL_SET,
+                     gensio.GENSIO_ACONTROL_SER_FLUSH,
+                     "recv", h, -1)
+        if h.wait_timeout(1000) == 0:
+            raise Exception("Timeout waiting for client flush response")
+    except:
+        utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
+        raise
+    utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
+    print("  Success!")
+    return
+
+test_flush()
+
 def test_modemstate():
     config = ("connection: &con",
               "  accepter: telnet(rfc2217),tcp,3023",
@@ -441,82 +507,97 @@ def test_modemstate():
     o = utils.o
     ser2net, io1, io2 = utils.setup_2_ser2net(o, config, io1str, io2str,
                                               do_io1_open = False)
-    set_remote_null_modem(utils.remote_id_int(io2), False);
-    set_remote_modem_ctl(utils.remote_id_int(io2), (SERIALSIM_TIOCM_CAR |
-                                           SERIALSIM_TIOCM_CTS |
-                                           SERIALSIM_TIOCM_DSR |
-                                           SERIALSIM_TIOCM_RNG) << 16)
+    try:
+        set_remote_null_modem(utils.remote_id_int(io2), False);
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             (SERIALSIM_TIOCM_CAR |
+                              SERIALSIM_TIOCM_CTS |
+                              SERIALSIM_TIOCM_DSR |
+                              SERIALSIM_TIOCM_RNG) << 16)
 
-    io1.handler.set_expected_modemstate(0)
-    io1.open_s()
-    io1.read_cb_enable(True);
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 1" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(0)
+        io1.open_s()
+        io1.read_cb_enable(True);
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 1" %
+                            ("test dtr", io1.handler.name))
 
-    io2.read_cb_enable(True);
+        io2.read_cb_enable(True);
 
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD)
-    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_CAR << 16) |
-                                           SERIALSIM_TIOCM_CAR))
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 2" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD)
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             ((SERIALSIM_TIOCM_CAR << 16) |
+                              SERIALSIM_TIOCM_CAR))
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 2" %
+                            ("test dtr", io1.handler.name))
 
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR)
-    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_DSR << 16) |
-                                           SERIALSIM_TIOCM_DSR))
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 3" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD |
+            gensio.SERGENSIO_MODEMSTATE_DSR)
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             ((SERIALSIM_TIOCM_DSR << 16) |
+                              SERIALSIM_TIOCM_DSR))
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 3" %
+                            ("test dtr", io1.handler.name))
 
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR |
-                                        gensio.SERGENSIO_MODEMSTATE_CTS)
-    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_CTS << 16) |
-                                           SERIALSIM_TIOCM_CTS))
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 4" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD |
+            gensio.SERGENSIO_MODEMSTATE_DSR |
+            gensio.SERGENSIO_MODEMSTATE_CTS)
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             ((SERIALSIM_TIOCM_CTS << 16) |
+                              SERIALSIM_TIOCM_CTS))
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 4" %
+                            ("test dtr", io1.handler.name))
 
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR |
-                                        gensio.SERGENSIO_MODEMSTATE_CTS |
-                                        gensio.SERGENSIO_MODEMSTATE_RI)
-    set_remote_modem_ctl(utils.remote_id_int(io2), ((SERIALSIM_TIOCM_RNG << 16) |
-                                           SERIALSIM_TIOCM_RNG))
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 5" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD |
+            gensio.SERGENSIO_MODEMSTATE_DSR |
+            gensio.SERGENSIO_MODEMSTATE_CTS |
+            gensio.SERGENSIO_MODEMSTATE_RI)
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             ((SERIALSIM_TIOCM_RNG << 16) |
+                              SERIALSIM_TIOCM_RNG))
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 5" %
+                            ("test dtr", io1.handler.name))
 
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED)
-    set_remote_modem_ctl(utils.remote_id_int(io2), (SERIALSIM_TIOCM_CAR |
-                                           SERIALSIM_TIOCM_CTS |
-                                           SERIALSIM_TIOCM_DSR |
-                                           SERIALSIM_TIOCM_RNG) << 16)
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 6" %
-                        ("test dtr", io1.handler.name))
-
-    io1.handler.set_expected_modemstate(gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
-                                        gensio.SERGENSIO_MODEMSTATE_CD |
-                                        gensio.SERGENSIO_MODEMSTATE_DSR |
-                                        gensio.SERGENSIO_MODEMSTATE_CTS)
-    set_remote_null_modem(utils.remote_id_int(io2), True);
-    if (io1.handler.wait_timeout(2000) == 0):
-        raise Exception("%s: %s: Timed out waiting for modemstate 7" %
-                        ("test dtr", io1.handler.name))
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_RI_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED)
+        set_remote_modem_ctl(utils.remote_id_int(io2),
+                             (SERIALSIM_TIOCM_CAR |
+                              SERIALSIM_TIOCM_CTS |
+                              SERIALSIM_TIOCM_DSR |
+                              SERIALSIM_TIOCM_RNG) << 16)
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 6" %
+                            ("test dtr", io1.handler.name))
 
+        io1.handler.set_expected_modemstate(
+            gensio.SERGENSIO_MODEMSTATE_CD_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_DSR_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CTS_CHANGED |
+            gensio.SERGENSIO_MODEMSTATE_CD |
+            gensio.SERGENSIO_MODEMSTATE_DSR |
+            gensio.SERGENSIO_MODEMSTATE_CTS)
+        set_remote_null_modem(utils.remote_id_int(io2), True);
+        if (io1.handler.wait_timeout(2000) == 0):
+            raise Exception("%s: %s: Timed out waiting for modemstate 7" %
+                            ("test dtr", io1.handler.name))
+    except:
+        utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
+        raise
     utils.finish_2_ser2net(ser2net, io1, io2, handle_except = False)
     print("  Success!")
     return
