@@ -102,6 +102,7 @@ class HandleData:
         self.expecting_modemstate = False
         self.expecting_modemstate_mask = False
         self.expecting_linestate = False
+        self.expecting_flush = False
         self.expected_server_cb = None
         self.expected_server_value = 0
         self.expected_server_return = 0
@@ -351,6 +352,24 @@ class HandleData:
         self.expected_linestate = linestate
         return
 
+    def flush(self, io, val):
+        if (not self.expecting_flush):
+            if (debug or self.debug):
+                print("Got unexpected flush %x" % val)
+            return
+        if (val != self.expected_flush):
+            raise HandlerException("%s: Expecting flush %d, got %d" %
+                                   (self.name, self.expected_flush,
+                                    val))
+        self.expecting_flush = False
+        self.waiter.wake()
+        return
+
+    def set_expected_flush(self, flush):
+        self.expecting_flush = True
+        self.expected_flush = flush
+        return
+
     def set_expected_server_cb(self, name, value, retval):
         self.expected_server_cb = name
         self.expected_server_value = value
@@ -713,8 +732,8 @@ def test_write_drain(io1, io2, data, timeout = 1000):
     If the transfer does not complete by "timeout" milliseconds, raise
     an exception.
     """
-    io1.handler.set_write_data(data, close_on_done = True)
     io2.handler.set_compare(data)
+    io1.handler.set_write_data(data, close_on_done = True)
     if (io1.handler.wait_timeout(timeout) == 0):
         raise Exception(("%s: %s: " % ("test_dataxfer", io1.handler.name)) +
                         ("Timed out waiting for write completion at byte %d" %
